from django.contrib import admin

from voice.models import Feature, Vote

class FeatureAdmin(admin.ModelAdmin):
    readonly_fields = ('created',)

class VoteAdmin(admin.ModelAdmin):
    readonly_fields = ('feature', 'used_facebook', 'used_twitter', 'voter',)

admin.site.register(Vote, VoteAdmin)
admin.site.register(Feature, FeatureAdmin)



from django.shortcuts import render_to_response
from django.template import RequestContext
from django.http import HttpResponse, HttpResponseRedirect, \
        Http404, HttpResponseNotModified
from django.contrib import messages
from django.db.models import Count

from voice import settings
from voice.models import Feature
from voice.forms import VoteForm, FeatureForm 

def index(request):
    feature_id = None
    form = VoteForm()

    sort, features = features_by_sort(request.GET.get('sort', None))

    grouped = []
    group = []
    for i, feature in enumerate(features):
        group.append(feature)
        if (i + 1) % 4 == 0:
            grouped.append(group)
            group = []
    if len(group) > 0:
        grouped.append(group)

    context = RequestContext(request, {
        'grouped_features': grouped,
        'form': form,
        'feature_id': feature_id,
        'request': request,
        'sort': sort,
        })

    return render_to_response('voice/index.html', context)

def feature(request, feature_id):
    feature = Feature.objects.get(id=feature_id)

    if request.method == 'POST':
        form = VoteForm(request.POST)
        if form.is_valid():
            vote = form.save(commit=False)
            vote.feature = feature
            vote.save()
            messages.success(request, 'Vote successfully submitted!')
    else:
        form = VoteForm()

    context = RequestContext(request, {
        'form': form,
        'request': request,
        'feature': feature,
        })

    return render_to_response('voice/feature.html', context)

def new_feature(request):
    if request.method == 'POST':
        form = FeatureForm(request.POST)
        if form.is_valid():
            feature = form.save(commit=False)
            feature.votes_needed = settings.DEFAULT_VOTES_NEEDED
            feature.save()
            messages.success(request, 'Thanks for requesting a feature!')
            form = FeatureForm()
    else:
        form = FeatureForm()

    context = RequestContext(request, {
        'form': form,
        })
    return render_to_response('voice/new_feature.html', context)

def admin(request):
    sort, features = features_by_sort(request.GET.get('sort', None))
    return render_to_response('voice/admin/index.html', {
        'request': request,
        'features': features,
        'sort': sort,
        })

def features_by_sort(sort):
    if sort == 'newest':
        features = Feature.objects.filter(state='V').order_by('-created')
    elif sort == 'in-progress':
        features = Feature.objects.filter(state='W')
    elif sort == 'finished':
        features = Feature.objects.filter(state='F')
    else:
        sort = 'popular'
        features = Feature.objects.filter(state='V').annotate(
                Count('votes')).order_by('-votes__count')
    return (sort, features)


def static_media(request, path):
    """
    Taken from django-sentry:
    http://github.com/dcramer/django-sentry

    Serve static files below a given point in the directory structure.
    """
    from django.utils.http import http_date
    from django.views.static import was_modified_since
    import mimetypes
    import os.path
    import posixpath
    import stat
    import urllib

    document_root = os.path.join(settings.ROOT, 'static')
    print document_root

    path = posixpath.normpath(urllib.unquote(path))
    path = path.lstrip('/')
    newpath = ''
    for part in path.split('/'):
        if not part:
            # Strip empty path components.
            continue
        drive, part = os.path.splitdrive(part)
        head, part = os.path.split(part)
        if part in (os.curdir, os.pardir):
            # Strip '.' and '..' in path.
            continue
        newpath = os.path.join(newpath, part).replace('\\', '/')
    if newpath and path != newpath:
        return HttpResponseRedirect(newpath)
    fullpath = os.path.join(document_root, newpath)
    if os.path.isdir(fullpath):
        raise Http404("Directory indexes are not allowed here.")
    if not os.path.exists(fullpath):
        raise Http404('"%s" does not exist' % fullpath)
    # Respect the If-Modified-Since header.
    statobj = os.stat(fullpath)
    mimetype = mimetypes.guess_type(fullpath)[0] or 'application/octet-stream'
    if not was_modified_since(request.META.get('HTTP_IF_MODIFIED_SINCE'),
                              statobj[stat.ST_MTIME], statobj[stat.ST_SIZE]):
        return HttpResponseNotModified(mimetype=mimetype)
    contents = open(fullpath, 'rb').read()
    response = HttpResponse(contents, mimetype=mimetype)
    response["Last-Modified"] = http_date(statobj[stat.ST_MTIME])
    response["Content-Length"] = len(contents)
    return response



from django.conf.urls.defaults import *

urlpatterns = patterns('voice',
        url(r'^static/(?P<path>.*)$', 'views.static_media',
            name='voice-media'),

        url(r'^$', 'views.index', name='voice-index'),
        url(r'^features/(?P<feature_id>\d+)$', 'views.feature',
        name='voice-feature'),
        url(r'^features/new/', 'views.new_feature', name='voice-new-feature'),
        url(r'^admin/', 'views.admin', name='voice-admin'),
        )



# Django settings for example_project project.
import sys
import os.path

PROJECT_ROOT = os.path.dirname(__file__)

sys.path.insert(0, os.path.abspath(os.path.join(PROJECT_ROOT, '..')))

DEBUG = True
TEMPLATE_DEBUG = DEBUG

ADMINS = (
    # ('Your Name', 'your_email@example.com'),
)

MANAGERS = ADMINS

DATABASE_ENGINE = 'sqlite3'
DATABASE_NAME = 'voice.sqlite'

TIME_ZONE = 'America/Chicago'

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = 'en-us'

SITE_ID = 1

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = True

# If you set this to False, Django will not format dates, numbers and
# calendars according to the current locale
USE_L10N = True

MEDIA_ROOT = ''

MEDIA_URL = ''

STATIC_ROOT = ''

# URL prefix for static files.
STATIC_URL = '/static/'

# URL prefix for admin static files -- CSS, JavaScript and images.
# Make sure to use a trailing slash.
ADMIN_MEDIA_PREFIX = '/static/admin/'

STATICFILES_DIRS = (
    # Put strings here, like "/home/html/static" or "C:/www/django/static".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
)

STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
)

SECRET_KEY = 'k7jlriij@o)^5mfip-7au2@b)mjzrz_pt=sz0eez0k!=*w5wa8'

TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
#     'django.template.loaders.eggs.Loader',
)

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
)

MESSAGE_STORAGE = 'django.contrib.messages.storage.session.SessionStorage'

ROOT_URLCONF = 'example_project.urls'

TEMPLATE_DIRS = (
    os.path.join(PROJECT_ROOT, 'templates'),
)

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.admin',
    'voice',
    'south',
)

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'handlers': {
        'mail_admins': {
            'level': 'ERROR',
            'class': 'django.utils.log.AdminEmailHandler'
        }
    },
    'loggers': {
        'django.request': {
            'handlers': ['mail_admins'],
            'level': 'ERROR',
            'propagate': True,
        },
    }
}



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'Request'
        db.create_table('voice_request', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('state', self.gf('django.db.models.fields.CharField')(max_length=1)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=50)),
            ('description', self.gf('django.db.models.fields.TextField')(max_length=140)),
            ('votes_needed', self.gf('django.db.models.fields.IntegerField')()),
            ('created', self.gf('django.db.models.fields.DateField')(auto_now_add=True, blank=True)),
        ))
        db.send_create_signal('voice', ['Request'])

        # Adding model 'Vote'
        db.create_table('voice_vote', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('request', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['voice.Request'])),
            ('voter', self.gf('django.db.models.fields.EmailField')(max_length=75)),
            ('used_twitter', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('used_facebook', self.gf('django.db.models.fields.BooleanField')(default=False)),
        ))
        db.send_create_signal('voice', ['Vote'])


    def backwards(self, orm):
        
        # Deleting model 'Request'
        db.delete_table('voice_request')

        # Deleting model 'Vote'
        db.delete_table('voice_vote')


    models = {
        'voice.request': {
            'Meta': {'object_name': 'Request'},
            'created': ('django.db.models.fields.DateField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'max_length': '140'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'state': ('django.db.models.fields.CharField', [], {'max_length': '1'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'votes_needed': ('django.db.models.fields.IntegerField', [], {})
        },
        'voice.vote': {
            'Meta': {'object_name': 'Vote'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'request': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['voice.Request']"}),
            'used_facebook': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'used_twitter': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'voter': ('django.db.models.fields.EmailField', [], {'max_length': '75'})
        }
    }

    complete_apps = ['voice']



from django import forms
from voice.models import Vote, Feature

class VoteForm(forms.ModelForm):

    class Meta:
        model = Vote
        exclude = ('used_twitter', 'used_facebook',)

    def clean(self):
        cleaned_data = self.cleaned_data
        feature_id = cleaned_data.get('feature')
        voter = cleaned_data.get('voter')

        vote = Vote.objects.filter(feature=feature_id, voter=voter)
        if vote.exists():
            error = '%s has already voted on this feature' % voter
            self._errors['feature'] = self._errors.get('feature', [])
            self._errors['feature'].append(error)
            raise forms.ValidationError(error)

        return cleaned_data

class FeatureForm(forms.ModelForm):

    description = forms.CharField(widget=forms.Textarea(
        attrs={'class':'xxlarge', 'rows':3}))
    voter = forms.EmailField()

    class Meta:
        model = Feature
        exclude = ('votes_needed', 'created', 'state',)



from django.db import models

class Feature(models.Model):
    FEATURE_STATES = (
            ('W', 'In-progress'),
            ('F', 'Finished'),
            ('V', 'Needs votes'),
            )
    state = models.CharField(max_length=1, choices=FEATURE_STATES, default='V')
    title = models.CharField(max_length=50)
    description = models.TextField(max_length=140)
    votes_needed = models.IntegerField()
    created = models.DateField(auto_now_add=True)

    def votes_left(self):
        votes_left = self.votes_needed - len(self.votes.all())
        if votes_left >= 0:
            return votes_left
        else:
            return 0

    def total_votes(self):
        return len(self.votes.all())

    def __unicode__(self):
        return self.title

class Vote(models.Model):
    feature = models.ForeignKey('Feature', related_name='votes')
    voter = models.EmailField()
    used_twitter = models.BooleanField(default=False)
    used_facebook = models.BooleanField(default=False)

    class Meta:
        unique_together = (('feature', 'voter'),)

    def __unicode__(self):
        return self.voter

    def save(self, *args, **kwargs):
        super(Vote, self).save(*args, **kwargs)
        self.feature.save()
        if self.feature.votes_left() == 0:
            self.feature.state = 'W'
            self.feature.save()



from django import template

register = template.Library()

@register.filter
def votes_left(feature):
    return feature.votes_left()

@register.filter
def state(feature):
    return feature.get_state_display()

@register.filter
def votes(feature):
    return feature.total_votes()



from django.utils import unittest
from django.db import IntegrityError
from voice.models import Feature, Vote

class FeatureTestCase(unittest.TestCase):
    def setUp(self):
        self.feature = Feature.objects.create(title='Feature',
                description='This is a feature', votes_needed=350)

    def testString(self):
        self.assertEqual('Feature', str(self.feature))

    def testDefaultState(self):
        self.assertEqual('V', self.feature.state)

    def testChangesState(self):
        for i in range(0, self.feature.votes_needed):
            voter = 'voter%d@domain.com' % i
            Vote.objects.create(feature=self.feature, voter=voter)
        self.assertEqual('W', self.feature.state)

    def testVoting(self):
        Vote.objects.create(feature=self.feature, voter='v@d.com')
        self.assertEqual(self.feature.votes.count(), 1)

class VoteTestCase(unittest.TestCase):
    def setUp(self):
        self.feature = Feature.objects.create(title='Feature',
                description='This is a feature', votes_needed=350)

    def testString(self):
        voter = 'v@d.com'
        vote = Vote.objects.create(feature=self.feature, voter=voter)
        self.assertEqual(voter, str(vote))

    def testDisallowDuplicates(self):
        voter = 'v@d.com'
        vote = Vote.objects.create(feature=self.feature, voter=voter)
        self.assertRaises(IntegrityError, Vote.objects.create,
                feature=self.feature, voter=voter)

    def testDecreasesVotesNeeded(self):
        for i in range(0, self.feature.votes_needed):
            voter = 'voter%d@domain.com' % i
            vote = Vote.objects.create(feature=self.feature, voter=voter)

        self.assertEqual(self.feature.votes_left(), 0)

        for i in range(self.feature.votes_needed + 1,
                self.feature.votes_needed * 2):
            voter = 'voter%d@domain.com' % i
            vote = Vote.objects.create(feature=self.feature, voter=voter)

        self.assertEqual(self.feature.votes_left(), 0)



from django.conf.urls.defaults import patterns, include, url

from django.contrib import admin
admin.autodiscover()

urlpatterns = patterns('',
    url(r'^admin/', include(admin.site.urls)),
    url(r'^voice/', include('voice.urls')),
)



#!/usr/bin/env python

try:
    from setuptools import setup, find_packages
except ImportError:
    from ez_setup import use_setuptools
    use_setuptools()
    from setuptools import setup, find_packages

tests_require = [
    'Django>=1.3.1',
]

setup(
    name='Voice',
    version='0.1.0',
    author='Jarod Luebbert',
    author_email='jarodluebbert@gmail.com',
    url='http://github.com/jarodl/voice',
    description='A voting app for letting users request features.',
    packages=find_packages(exclude=("example_project",)),
    zip_safe=False,
    install_requires=[
        'Django>=1.3.1',
        'South',
        ],
    tests_require=tests_require,
    extras_require={'test':tests_require},
    test_suite = 'nose.collector',
    include_package_data=True,
    classifiers=[
      'Framework :: Django',
      'Intended Audience :: Developers',
      'Intended Audience :: End Users/Desktop',
      'Operating System :: OS Independent',
    ],
)



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding unique constraint on 'Vote', fields ['voter', 'request']
        db.create_unique('voice_vote', ['voter', 'request_id'])


    def backwards(self, orm):
        
        # Removing unique constraint on 'Vote', fields ['voter', 'request']
        db.delete_unique('voice_vote', ['voter', 'request_id'])


    models = {
        'voice.request': {
            'Meta': {'object_name': 'Request'},
            'created': ('django.db.models.fields.DateField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'max_length': '140'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'state': ('django.db.models.fields.CharField', [], {'default': "'V'", 'max_length': '1'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'votes_needed': ('django.db.models.fields.IntegerField', [], {})
        },
        'voice.vote': {
            'Meta': {'unique_together': "(('request', 'voter'),)", 'object_name': 'Vote'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'request': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'votes'", 'to': "orm['voice.Request']"}),
            'used_facebook': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'used_twitter': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'voter': ('django.db.models.fields.EmailField', [], {'max_length': '75'})
        }
    }

    complete_apps = ['voice']
